import java.util.*;

/**
 * clause in Herbrand normaalvorm, dwz een conjunctie van atomen, zonder herhaling
 */
public class HerbrandClause implements Comparable<HerbrandClause> {

  /**
   * een Herbrand clause representeren we als gesorteerde verzameling van atomen, gerepresenteerd door hun index
   * de lege clause representeert waar
   */
  private SortedSet<Integer> indices;
  
  public HerbrandClause() { indices = new TreeSet<Integer>(); }
  
  public HerbrandClause(Integer index) { this(); indices.add(index); } 
  
  SortedSet<Integer> getIndices() { return indices; }
  
  public String toString() {
    String string = AbstractFormule.Connectief.WAAR.toString();
    int teller = 0;
    for (Integer index:indices)
      string = (teller == 0) ? "P"+index.toString() : string+" "+AbstractFormule.Connectief.EN.toString()+" P"+index.toString(); 
    return "("+string+")";
  }
  
  /**
   * @return vergelijking van deze clause met een andere, eerst op het aantal atomen, daarna op hun indices van klein naar groot
   */
  public int compareTo(HerbrandClause herbrandClause) {
    int vergelijk = new Integer(indices.size()).compareTo(new Integer(herbrandClause.getIndices().size()));
    Iterator<Integer> iterator = herbrandClause.getIndices().iterator();
    for (Integer index:indices) {
      if (vergelijk != 0) 
        return vergelijk;
      vergelijk = index.compareTo(iterator.next());
    }
    return vergelijk;
  }
  
  /**
   * @return waar als gelijk, afgeleid van vergelijking
   */
  public boolean equals(HerbrandClause herbrandClause) { return compareTo(herbrandClause) == 0; }
  
  /**
   * @return vereniging van deze met andere clause (met verwijdering van dubbele atomen)
   */
  public HerbrandClause vereniging(HerbrandClause herbrandClause) {
    HerbrandClause vereniging = new HerbrandClause();
    vereniging.getIndices().addAll(indices);
    vereniging.getIndices().addAll(herbrandClause.getIndices());
    return vereniging;
  }
  
  /**
   * @return normale formule
   */
  public AbstractFormule toFormule() {
    AbstractFormule formule = new Waar();
    int teller = 0;
    for (Integer index:getIndices()) 
      formule = (teller++ == 0) ? new Atoom(index) : new En(formule,new Atoom(index));
    return formule;
  }
}
