import java.util.*;

/**
 * formule in Herbrand normaalvorm, dwz een exof van conjuncties van atomen, zonder herhaling in beide
 */
public class HerbrandFormule implements Comparable<HerbrandFormule> {
  
  /**
   * een Herbrand formule representeren we als gesorteerde verzameling van Herbrand clauses
   * de lege formule representeert onwaar
   */
  private SortedSet<HerbrandClause> clauses;
    
  public HerbrandFormule() { clauses = new TreeSet<HerbrandClause>(); }
  
  public HerbrandFormule(HerbrandClause herbrandClause) { this(); clauses.add(herbrandClause); } 

  public HerbrandFormule(Integer index) { this(new HerbrandClause(index)); } 
  
  SortedSet<HerbrandClause> getClauses() { return clauses; }
  
  public String toString() {
    String string = AbstractFormule.Connectief.ONWAAR.toString();
    int teller = 0;
    for (HerbrandClause clause:clauses)
      string = (teller == 0) ? clause.toString() : string+" "+AbstractFormule.Connectief.EXOF.toString()+" "+clause.toString(); 
    return string;
  }
  
  /**
   * @return vergelijking van deze formule met een andere, eerst op het aantal clauses, daarna op de clauses van klein naar groot
   * (niet gebruikt; alleen vanwege analogie met clauses; en omdat het kan)
   */
  public int compareTo(HerbrandFormule herbrandFormule) {
    int vergelijk = new Integer(getClauses().size()).compareTo(new Integer(herbrandFormule.getClauses().size()));
    Iterator<HerbrandClause> iterator = herbrandFormule.getClauses().iterator();
    for (HerbrandClause clause:clauses) {
      if (vergelijk != 0) 
        return vergelijk;
      vergelijk =  clause.compareTo(iterator.next());
    }
    return vergelijk;
  }
  
  /**
   * @return waar als gelijk, afgeleid van vergelijking
   */
  public boolean equals(HerbrandFormule herbrandFormule) { return compareTo(herbrandFormule) == 0; }

  /**
   * @return state geupdatet met ware formule
   */  
  public void waar() { clauses.add(new HerbrandClause()); }

  /**
   * @return state geupdatet met exof met formule
   */
  public void exof(HerbrandFormule herbrandFormule) {
    for (HerbrandClause clause:herbrandFormule.getClauses())
      if (clauses.contains(clause)) {
        clauses.remove(clause); // als clause al aanwezig dan weghalen 
      } else {
        clauses.add(clause); // als clause nog niet aanwezig dan toevoegen
      }
  }
  
  /**
   * @return state geupdatet met conjunctie met formule
   */
  public void en(HerbrandFormule herbrandFormule) {
    SortedSet<HerbrandClause> herbrandClauses = clauses;
    clauses = new TreeSet<HerbrandClause>();
    for (HerbrandClause clause:herbrandClauses) 
      for (HerbrandClause distributieClause:herbrandFormule.getClauses())
        exof(new HerbrandFormule(clause.vereniging(distributieClause))); // distribueer ieder van de clauses van formule over deze clauses
  }

  /**
   * @return normale formule
   */
  public AbstractFormule toFormule() {
    AbstractFormule formule = new Onwaar();
    int teller = 0;
    for (HerbrandClause clause:clauses)
      formule = (teller++ == 0) ? clause.toFormule() : new Exof(formule,clause.toFormule());
    return formule;
  }
}