public interface Formule {
  
  /**
   * @return geeft de formule als String weer waarbij de haakjesconventie voor
   *         formules gevolgd is om zoveel mogelijk haakjes weg te laten (zonder ambiguiteit te creeren).
   *         zie hoofdstuk 4 van de <a href="http://www.annotatiesysteem.nl/index.php?m=document.view&id=11440&demo=1">syllabus</a>
   *         waarbij we de exof behandelen als de en en de of wbt prioriteit 
   */
  public String toString();
  
  /**
   * @param bedeling waarbij een negatief getal, b.v. -5, staat voor
   *        het toekennen van false aan dat atoom (hier atoom p5), en een positief getal b.v. 7 staat
   *        voor het toekennen van true aan dat atoom (hier atoom p7)
   * @return true als formule voor gegeven bedeling waar is, false als onwaar en er wordt
   *        een exception gegooid als de waarheidswaarde niet bepaald kan worden op basis
   *        van de gegeven bedeling. hierbij weten we dat een of waar is 
   *        als we weten dat een van beide argumenten waar is (zonder de waarheidswaarde van
   *        het andere te kennen), en duaal weten we dat een en onwaar is
   *        als we weten dat een van beide argumenten onwaar is (zonder de waarheidswaarde van
   *        het andere te kennen).
   *        
   */
  public boolean waarde(int[] bedeling) throws OnvoldoendeBedeling;

  /**
   * @return als de formule vervulbaar is wordt een bedeling die de formule waar maakt
   *         teruggeven (zie voor de codering van bedelingen het commentaar van waarde), en anders null.
   *         
   */ 
  public int[] vervulbaar();
  
  /**
   * @return geeft een met de formule equivalente formule die in zogeheten Herbrand normaalvorm is
   *         dat wil zeggen de formule wordt eerst geheel uitgedrukt met behulp van alleen 
   *         waar (1), onwaar (0), en (.), exof (+). daarna wordt de resulterende formule
   *         zoveel mogelijk vereenvoudigd met behulp van Herbrand's regels:
   *         <ul>
   *         <li> A + 0        ~>  A
   *         <li> A + A        ~>  0
   *         <li> A . 0        ~>  0
   *         <li> A . 1        ~>  A
   *         <li> A . A        ~>  A
   *         <li> A . (B + C)  ~>  (A . B) + (A . C)
   *         </ul>
   *         waarbij de regels op commutativiteit en associativiteit van en en exof na, worden toegepast.
   *         op commutativiteit na wil zeggen op volgorde na. 
   *         b.v. de eerste regel is ook van toepassing op 0 + A, want op A + 0.
   *         op associativiteit na wil zeggen op haakjes zetten na. 
   *         b.v. de eerste regel is ook van toepassing op (B + A) + 0, want op B + (A + 0)).
   *         (dus, op commutativeit en associativiteit na zijn (A + B) + C en (C + A) + B hetzelfde)
   *
   *         om deze methode te testen kunt u gebruikmaken van het feit dat een
   *         formule een tautologie is dan en slechts dan als hij vereenvoudigd wordt tot waar (1).
   */
  public Formule herbrandNormaalvorm();
  
}